import logging
from urllib.parse import urlsplit

from har2locust.plugin import entriesprocessor_with_args


@entriesprocessor_with_args  # use with-args version because it is executed last
def default_headers(entries: list[dict], _args):
    # calculate headers shared by all requests (same name and value)
    default_headers = None
    for e in entries:
        headers = e["request"]["headers"]
        if default_headers is None:
            default_headers = headers[:]
        else:
            for dh in default_headers[:]:
                if dh["name"] == "accept-language":
                    # accept-language is likely to be common among most requests so we never remove it from default headers
                    # (instead it will be overridden in the specific requests where it differs)
                    continue
                for h in headers:
                    if dh["name"] == h["name"]:
                        if dh["value"] != h["value"]:
                            logging.debug(
                                f"removed default header {dh['name']} with value {dh['value']} from default headers because it has different value in {e['request']['url']}"
                            )
                            default_headers.remove(dh)  # header has different value
                        break
                else:
                    logging.debug(
                        f"removed default header {dh['name']} with value {dh['value']} from default headers because it was not present in {e['request']['url']}"
                    )
                    default_headers.remove(dh)  # header not present
    if default_headers is None:
        default_headers = []

    default_headers.sort(key=lambda item: item["name"])

    urlparts = urlsplit(entries[0]["request"]["url"])
    host = f"{urlparts.scheme}://{urlparts.netloc}"
    for e in entries:
        e["request"]["url"] = e["request"]["url"].removeprefix(host)
        headers = e["request"]["headers"]
        for h in headers[:]:
            for dh in default_headers:
                if h["name"] == dh["name"] and h["value"] == dh["value"]:
                    headers.remove(dh)
        headers[:] = sorted(headers, key=lambda item: item["name"])

    return {"host": host, "default_headers": default_headers}



from locust import events, run_single_user, task

import os
import re

from locust_plugins.listeners import StopUserOnFail
from svs_locust import RestUser

os.environ["LOCUST_TEST_ENV"] = "itp1"
os.environ["LOCUST_TENANT"] = "lb"


@events.init.add_listener
def on_locust_init(environment, **_kwargs):
    import time

    environment.events.request.add_listener(lambda *args, **kw: time.sleep(0.1))
    StopUserOnFail(environment)


class reqres_in(RestUser):
    default_headers = {
        "accept": "*/*",
        "accept-encoding": "gzip, deflate, br, zstd",
        "accept-language": "sv,en;q=0.9",
        "cache-control": "no-cache",
        "content-type": "application/json",
        "pragma": "no-cache",
        "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/122.0.0.0 Safari/537.36",
    }

    @task
    def t(self):
        with self.rest("POST", "/api/register", json={"email": "eve.holt@reqres.in", "password": "pistol"}) as resp:
            job = re.findall('"token":"([^"]*)"', resp.text)[0] if resp.text else None
        with self.rest("PATCH", "/api/users/2", json={"name": "morpheus", "job": job}) as resp:
            pass


if __name__ == "__main__":
    run_single_user(reqres_in)



from locust import FastHttpUser, run_single_user, task


class login(FastHttpUser):
    host = "https://api.spela.test3.svenskaspel.se"
    default_headers = {
        "accept-encoding": "gzip, deflate, br",
        "accept-language": "sv,en;q=0.9",
        "cache-control": "no-cache",
        "pragma": "no-cache",
        "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Safari/537.36",
    }

    @task
    def t(self):
        with self.rest(
            "POST",
            "/player/1/authenticate/testlogin",
            headers={"accept": "application/json, text/javascript, */*; q=0.01"},
            json={"personalId": "193804122491", "source": 3},
        ) as resp:
            pass
        with self.rest(
            "GET",
            "/player/1/customizedsettings?_=1636025335990",
            headers={"accept": "application/json, text/javascript, */*; q=0.01"},
        ) as resp:
            pass
        with self.client.request(
            "GET",
            "https://spela.test3.svenskaspel.se/",
            headers={
                "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9"
            },
            catch_response=True,
        ) as resp:
            pass
        with self.client.request(
            "GET",
            "https://spela.test3.svenskaspel.se/logga-in/uppdaterade-villkor?returnUrl=%2F",
            headers={
                "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9"
            },
            catch_response=True,
        ) as resp:
            pass
        with self.rest(
            "POST", "/player/1/terms", headers={"accept": "application/json, text/javascript, */*; q=0.01"}, json={}
        ) as resp:
            pass
        with self.rest(
            "GET",
            "/player/1/info?include=accountBalance&_=1636025343876",
            headers={"accept": "application/json, text/javascript, */*; q=0.01"},
        ) as resp:
            pass


if __name__ == "__main__":
    run_single_user(login)



import subprocess

from har2locust.plugin import outputstringprocessor


@outputstringprocessor
def ruff(py: str):
    p = subprocess.Popen(["ruff", "format", "-"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, text=True)
    assert p.stdin  # keep linter happy
    p.stdin.write(py)
    stdout, _stderr = p.communicate()
    assert (
        not p.returncode
    ), f"Ruff failed to format the output - perhaps your template is broken? unformatted output was: {py}"
    p = subprocess.Popen(
        [
            "ruff",
            "check",
            "--fix",
            "-q",
            "--exit-zero",  # ignore linting fails, we're most interested in applying fixes, and it is up to the user to examin errors
            "-",
        ],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        text=True,
    )
    assert p.stdin  # keep linter happy
    p.stdin.write(stdout)
    stdout, stderr = p.communicate()
    assert (
        not p.returncode
    ), f"Ruff failed to check/fix the output - perhaps your template is broken? unformatted output was: {stdout}"

    # for some reason the subprocess returns an extra newline, get rid of that
    return stdout[:-1]



import ast
from argparse import Namespace
from typing import Callable, Optional

# For examples of how to write a plugin, see default_plugins/ or tests/plugin_example.py
# The processors allow you to interact with your recording at various stages,


# immediately after reading the HAR JSON
class entriesprocessor:
    processors: list[Callable[[list[dict]], Optional[dict]]] = []

    def __init__(self, func: Callable[[list[dict]], Optional[dict]]):
        self.processors.append(func)


# immediately after reading the HAR JSON, with access to command line arguments
class entriesprocessor_with_args:
    processors: list[Callable[[list[dict], Namespace], Optional[dict]]] = []

    def __init__(self, func: Callable[[list[dict], Namespace], Optional[dict]]):
        self.processors.append(func)


# after rendering template, working with the syntax tree
class astprocessor:
    processors: list[Callable[[ast.Module, dict], None]] = []

    def __init__(self, func: Callable[[ast.Module, dict], None]):
        self.processors.append(func)


# just before printing output, working with the string output
class outputstringprocessor:
    processors: list[Callable[[str], str]] = []

    def __init__(self, func: Callable[[str], str]):
        self.processors.append(func)



import json

from har2locust.plugin import entriesprocessor


@entriesprocessor
def rest(entries: list[dict]):
    for e in entries:
        for h in e["response"]["headers"]:
            if h["name"].lower() == "content-type":
                if h["value"].startswith("application/json"):
                    r = e["request"]
                    if (
                        "postData" not in r  # not a post, ok for .rest
                        or r["postData"]["mimeType"] == "application/json"  # json payload, also ok
                    ):
                        # logging.debug(f"{r['url']} is a rest request!")
                        if "postData" in r and "text" in r["postData"]:
                            r["postData"]["text"] = json.loads(r["postData"]["text"])
                        r["fname"] = "rest"
                        r["extraparams"] = []  # catch_response=True is already the default for .rest()


# The following is no longer needed, now that rest-method is a part of FastHttpUser, but can still serve as a good example:
# from ast import *
# from har2locust.plugin import astprocessor
#
# @astprocessor
# def process_ast(tree: Module, values: dict):
#     class Transformer(NodeTransformer):
#         def visit_ClassDef(self, node: ClassDef) -> ClassDef:
#             node.bases[0] = Name("RestUser")
#             self.generic_visit(node)
#             return node

#         def visit_ImportFrom(self, node: ImportFrom) -> ImportFrom:
#             if node.names[0].name == "FastHttpUser":
#                 node.module = "locust_plugins.users"
#                 node.names[0].name = "RestUser"
#             self.generic_visit(node)
#             return node

#         def visit_Module(self, node: Module) -> Module:
#             node.body = (
#                 parse(
#                     """
# from locust import events
# from locust_plugins.listeners import RescheduleTaskOnFail
# """
#                 ).body
#                 + node.body
#                 + parse(
#                     f"""
# @events.init.add_listener
# def on_locust_init(environment, **_kwargs):
#     RescheduleTaskOnFail(environment)
# if __name__ == "__main__":
#     run_single_user({values['class_name']})
# """,
#                 ).body
#             )
#             self.generic_visit(node)
#             return node

#     Transformer().visit(tree)



import ast
import importlib
import json
import logging
import os
import pathlib
import sys
from argparse import Namespace

import jinja2

from .argument_parser import get_parser
from .plugin import astprocessor, entriesprocessor, entriesprocessor_with_args, outputstringprocessor


def __main__(arguments=None):
    args = get_parser().parse_args(arguments)
    logging.basicConfig(level=args.loglevel.upper())
    load_plugins(
        args.plugins.split(",") if args.plugins else [], args.disable_plugins.split(",") if args.disable_plugins else []
    )
    har_path = pathlib.Path(args.input)
    name = har_path.stem.replace("-", "_").replace(".", "_")  # build class name from filename
    with open(har_path, encoding="utf8") as f:
        har = json.load(f)
    logging.debug(f"loaded {har_path}")

    template_values = process(har, args)
    template_values["class_name"] = name
    py = render(args.template, template_values)
    print(py)


def load_plugins(plugins: list[str] = [], disable_plugins: list[str] = []):
    package_root_dir = pathlib.Path(__file__).parents[1]
    plugin_dir = package_root_dir / "har2locust/default_plugins"
    default_plugins = [str(d.relative_to(package_root_dir)) for d in plugin_dir.glob("*.py")]
    default_plugins.sort()  # ensure deterministic ordering
    default_and_extra_plugins = default_plugins + plugins

    plugins_copy = default_and_extra_plugins.copy()
    for dp in disable_plugins:
        for p in default_and_extra_plugins[:]:
            if p.endswith(dp):
                default_and_extra_plugins.remove(p)
                logging.debug("Disabled plugin: " + p)
                break
        else:
            raise Exception(f"Tried to disable unknown plugin: {dp}. Known plugins are {plugins_copy}")

    sys.path.append(os.path.curdir)  # accept plugins by relative path
    for plugin in default_and_extra_plugins:
        # in  windows OS   print(plugin) echo  har2locust\default_plugins\1_resourcefilter.py
        import_path = plugin.replace("/", ".").replace("\\", ".").rstrip(".py")
        importlib.import_module(import_path)
        logging.debug("imported " + import_path)
    logging.debug(f"loaded plugins {default_and_extra_plugins}")


# process har dictionary and return a dict of values to render
def process(har: dict, args: Namespace) -> dict:
    if har["log"]["version"] != "1.2":
        logging.warning(f"Untested har version {har['log']['version']}")
    entries = har["log"]["entries"]
    logging.debug(f"found {len(entries)} entries")

    for e in entries:
        # set defaults
        e["request"]["fname"] = "client.request"
        e["request"]["extraparams"] = [("catch_response", True)]
    values = {"entries": entries}
    for p in entriesprocessor.processors:
        values |= p(entries) or {}
        # logging.debug(f"{len(entries)} entries after applying {p.__name__}")
    for p in entriesprocessor_with_args.processors:
        values |= p(entries, args) or {}
        # logging.debug(f"{len(entries)} entries after applying with_args {p.__name__}")
    logging.debug(f"{len(entries)} entries after applying entriesprocessors")

    return values


def render(name: str, values: dict) -> str:
    logging.debug(f'about to load template "{name}"')
    if pathlib.Path(name).exists():
        template_path = pathlib.Path(name)
    else:
        template_path = pathlib.Path(__file__).parents[0] / name
        if not template_path.exists():
            raise Exception(f"Template {name} does not exist, neither in current directory nor as built in")

    template_dir = template_path.parents[0]

    env = jinja2.Environment(loader=jinja2.FileSystemLoader(template_dir))
    template = env.get_template(template_path.name)
    logging.debug("template loaded")

    py = template.render(values)
    logging.debug("template rendered")

    try:
        tree = ast.parse(py, type_comments=True)
    except ValueError as e:
        logging.debug(py)
        levelmessage = " (set log level DEBUG to see the whole output)" if logging.DEBUG < logging.root.level else ""
        logging.error(f"{e} when parsing rendered template{levelmessage}")
        raise
    except SyntaxError as e:
        logging.debug(py)
        levelmessage = " (set log level DEBUG to see the whole output)" if logging.DEBUG < logging.root.level else ""
        logging.error(f"{e.msg} when parsing rendered template{levelmessage}")
        raise

    for p in astprocessor.processors:
        p(tree, values)
    py = ast.unparse(ast.fix_missing_locations(tree))
    logging.debug("astprocessors applied")

    for p in outputstringprocessor.processors:
        py = p(py)
    logging.debug("outputstringprocessors applied")

    return py


if __name__ == "__main__":
    __main__()



from locust import events, run_single_user, task

import os

from locust_plugins.listeners import StopUserOnFail
from svs_locust import RestUser

os.environ["LOCUST_TEST_ENV"] = "itp1"
os.environ["LOCUST_TENANT"] = "lb"


@events.init.add_listener
def on_locust_init(environment, **_kwargs):
    import time

    environment.events.request.add_listener(lambda *args, **kw: time.sleep(0.1))
    StopUserOnFail(environment)


class login(RestUser):
    default_headers = {
        "accept-encoding": "gzip, deflate, br",
        "accept-language": "sv,en;q=0.9",
        "cache-control": "no-cache",
        "pragma": "no-cache",
        "user-agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Safari/537.36",
    }

    @task
    def t(self):
        self.customer = next(self.customer_iterator)
        self.auth()
        with self.rest_(
            "GET", "/player/1/customizedsettings", headers={"accept": "application/json, text/javascript, */*; q=0.01"}
        ) as resp:
            pass
        with self.client.request(
            "GET",
            "https://spela.test3.svenskaspel.se/",
            headers={
                "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9"
            },
            catch_response=True,
        ) as resp:
            pass
        with self.client.request(
            "GET",
            "https://spela.test3.svenskaspel.se/logga-in/uppdaterade-villkor?returnUrl=%2F",
            headers={
                "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9"
            },
            catch_response=True,
        ) as resp:
            pass
        with self.rest(
            "POST", "/player/1/terms", headers={"accept": "application/json, text/javascript, */*; q=0.01"}, json={}
        ) as resp:
            pass
        with self.rest_(
            "GET",
            "/player/1/info?include=accountBalance",
            headers={"accept": "application/json, text/javascript, */*; q=0.01"},
        ) as resp:
            pass


if __name__ == "__main__":
    run_single_user(login)
